//
//  MapQuakesViewController.m
//  MapQuakes
//
//  Created by Screen Caster on 6/25/09.
//  Copyright Gala Factory Software LLC 2009. All rights reserved.
//

#import "MapQuakesViewController.h"
#import "Earthquake.h"
#import "EarthquakeParser.h"
#import "EarthquakeAnnotation.h"

@implementation MapQuakesViewController

@synthesize mapView = _mapView;

- (void)viewWillAppear:(BOOL)animated {
  [super viewWillAppear:animated];
}

- (void)addEarthquake:(Earthquake *)earthquake {
  if([self.mapView.userLocation.location getDistanceFrom:earthquake.location] < 250.0f * 5280.0f / 3.0f) {
    EarthquakeAnnotation *eqAnn = [EarthquakeAnnotation annotationWithEarthquake:earthquake];
    [self.mapView addAnnotation:eqAnn];
  }
}

- (void)recenterMap {
	NSArray *coordinates = [self.mapView valueForKeyPath:@"annotations.coordinate"];
	CLLocationCoordinate2D maxCoord = {-90.0f, -180.0f};
	CLLocationCoordinate2D minCoord = {90.0f, 180.0f};
	for(NSValue *value in coordinates) {
		CLLocationCoordinate2D coord = {0.0f, 0.0f};
		[value getValue:&coord];
		if(coord.longitude > maxCoord.longitude) {
			maxCoord.longitude = coord.longitude;
		}
		if(coord.latitude > maxCoord.latitude) {
			maxCoord.latitude = coord.latitude;
		}
		if(coord.longitude < minCoord.longitude) {
			minCoord.longitude = coord.longitude;
		}
		if(coord.latitude < minCoord.latitude) {
			minCoord.latitude = coord.latitude;
		}
	}
	MKCoordinateRegion region = {{0.0f, 0.0f}, {0.0f, 0.0f}};
	region.center.longitude = (minCoord.longitude + maxCoord.longitude) / 2.0;
	region.center.latitude = (minCoord.latitude + maxCoord.latitude) / 2.0;
	region.span.longitudeDelta = maxCoord.longitude - minCoord.longitude;
	region.span.latitudeDelta = maxCoord.latitude - minCoord.latitude;
	[self.mapView setRegion:region animated:YES];  
}

- (void)parserFinished {
  [self recenterMap];
}

- (MKAnnotationView *)mapView:(MKMapView *)mapView 
            viewForAnnotation:(id <MKAnnotation>)annotation {
  MKAnnotationView *view = nil;
  if(annotation != mapView.userLocation) {
		EarthquakeAnnotation *eqAnn = (EarthquakeAnnotation*)annotation;
		view = [self.mapView dequeueReusableAnnotationViewWithIdentifier:@"earthquakeLoc"];
		if(nil == view) {
			view = [[[MKPinAnnotationView alloc] initWithAnnotation:eqAnn
                                              reuseIdentifier:@"earthquakeLoc"] autorelease];
		}
		CGFloat magnituide = [eqAnn.earthquake.magnitude floatValue];
		if(magnituide >= 4.0f) {
			[(MKPinAnnotationView *)view setPinColor:MKPinAnnotationColorRed];
		} else if(magnituide >= 3.0f) {
			[(MKPinAnnotationView *)view setPinColor:MKPinAnnotationColorPurple];
		} else if(magnituide >= 2.0f) {
			[(MKPinAnnotationView *)view setPinColor:MKPinAnnotationColorGreen];
		}
		[(MKPinAnnotationView *)view setAnimatesDrop:YES];
		[view setCanShowCallout:YES];
    [view setRightCalloutAccessoryView:[UIButton buttonWithType:UIButtonTypeDetailDisclosure]];
  } else {
    EarthquakeParser *parser = [EarthquakeParser earthquakeParser];
    parser.delegate = self;
    [parser getEarthquakeData];
  }

  return view;
}

- (void)mapView:(MKMapView *)mapView 
 annotationView:(MKAnnotationView *)view 
calloutAccessoryControlTapped:(UIControl *)control {
  EarthquakeAnnotation *eqAnn = (EarthquakeAnnotation *)[view annotation];
  NSURL *url = [NSURL URLWithString:eqAnn.earthquake.detailsURL];
  [[UIApplication sharedApplication] openURL:url];
}

/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/


/*
// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
}
*/


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}

@end
